#!/bin/bash
set -ue

function usage {
    echo "Usage: $0 (theme) [deb|rpm]"
    echo "    - (theme) means the theme directory name - also used in package name"
    echo "      No spaces, please."

    exit 1
}

# check parameter
[ $# -eq 0 ] && usage
[ "$1" = "-h" -o "$1" = "-H" ] && usage

# identify OS type
# - check for existence of dpkg

packageType=''

if [[ $# -eq 2 ]] ;then
  if [ "$2" = "deb" -o "$2" = "rpm" ] ; then
      packageType=$2
  fi
else
    set +e
    type -t dpkg > /dev/null
    result=$?
    set -e
    if [ $result -eq 0 ]
    then
        packageType=deb
        echo "Packaging deb"
    else
        packageType=rpm
        echo "Packaging rpm"
    fi
fi

theme=$1

if [ ! -d "src/$theme" ]
then
    echo "Directory not found:  \"src/$theme\""
    echo
    usage
fi


# set variables
serverPath=etc/opt/scalix/web/swa/themes
here=`pwd`
packageRoot=scalix-swa-theme
packageName=$packageRoot-$theme
dist=$packageName-12.2.0

xmllint=$(type -P xmllint)

if [ -n "$xmllint" ]; then
    $xmllint --noout src/$theme/properties.xml
fi

if [ "$packageType" = "deb" ]
then
    # Modify directory name for debian naming requirements
    # this affects the name of the package created
    # "all" here means "runs on all architectures", like "noarch" in RPM names

    # Change LANG to C to guarantee order of A-Z in all LOCALEs
    # Uppercase letters not allowed in Debian package names
    # Underscore not allowed in main part of name in Debian package names
    dist=`echo $dist | LANG=C sed -e 's/\([A-Z]\)/\l\1/g' \
                                  -e 's/\(_\)/./g'`

    dist=${dist}_all

    # copy files to dist folder
    mkdir -p $dist/$serverPath
    cp -a src/$theme $dist/$serverPath/

    mkdir -p $dist/DEBIAN
    cp packages/deb/control $dist/DEBIAN/

    # Set the size in the control file
    cd $dist
    # Change LANG to C to guarantee order of 0-9 in all LOCALEs
    size=`du -ksc etc | grep total | LANG=C sed 's/\([0-9]\+\).*/\1/'`

    cd $here
    sed -i -e "s/@SIZE@/$size/" \
           -e "s/@THEME_NAME@/$theme/g" \
           $dist/DEBIAN/control

    # build package
    dpkg-deb --build $dist
    if [ -d "$dist" ]; then
        rm -rf "$dist"
    fi

elif [ "$packageType" = "rpm" ]
then
    # we require rpmbuild to be present so we can use it
    set +e
    type -t rpmbuild > /dev/null
    result=$?
    set -e
    if [ $result -eq 0 ]
    then
        echo "Required program: rpmbuild - available"
    else
        echo "Could not find required program rpmbuild."
        echo "Please install package 'rpm-build' or equivalent."
        echo "Unable to continue."
        exit 1
    fi

    spec=scalix-theme.spec
    build_dir="$here/build"

    if [ -d $build_dir ]; then
        rm -rf "$build_dir"
    fi

    mkdir -p "$build_dir"/{BUILD,RPMS,SOURCES,SPECS,SRPMS}
    spec_dir="$build_dir/SPECS/"

    # copy files to dist folder
    rm -rf $dist
    mkdir -p $dist/$serverPath/
    cp -a src/$theme/ $dist/$serverPath/

    cp $here/packages/rpm/$spec "$spec_dir/"

    # put the theme name in the spec file
    sed -i -e "s/@THEME_NAME@/$theme/g" "$spec_dir/$spec"

    # generate the list of files and put in the spec file
    filelist=`ls src/$theme/`
    theme_files=""
    for file in $filelist
    do
        theme_files="$theme_files/$serverPath/$theme/$file\n"
    done
    sed -i -e "s#@theme-files@#$theme_files#" "$spec_dir/$spec"

    # build tarball so "%setup -c" in spec file can unpack it in the right place
    # the "right place" seems to be
    cd $dist
    tarball=$here/swa-theme.tar.gz
    tar czf $tarball *
    cp $tarball "$build_dir/SOURCES/"
    cd $here

    # build package
    cd $dist
    eval "rpmbuild -ba --define '_topdir $build_dir' $spec_dir/$spec"
    cd $here

    # get package
    cp $build_dir/RPMS/**/$packageName*rpm .

    for i in $tarball $dist $build_dir
    do
        if [ -f "$i" -o -d "$i" ]; then
            rm -rf "$i"
        fi
    done
else
    echo "Something went wrong - [$packageType] should be rpm or deb"
    exit 2
fi

# report success
echo Packaging \"$theme\" locale in \"$packageType\" format
echo Complete.
